//////////////////////////////////////////////////////////////////////////////////
//																				//
//		Roller																	//
//		Test and Demo of RollAPI library										//
//		Copyright (C),1998 SoftLab-Nsk											//
//																				//
//		Author:		Serge Horun													//
//																				//
//		Last revised(86C):		Dec 25, 1998, Serge Horun						//
//																				//
//////////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "RollAPI.h"
#include "RollBase.h"

CImageDescPtrList g_plImages(50);
CRegionPtrList g_plRegions;
long g_DefaultBank = RollAPI_PlayDefaultB;
CRect g_rcScreen;

void InitScreenInfo()
{
	RollAPI_GetScreenSize(NULL, &g_rcScreen);
}

static void DestroyImageDesc(CImageADesc * pDesc);

static CImageADesc * CreateImageDesc(CString &name)
{
	BITMAPFILEHEADER * pFileData = NULL;
	BITMAPINFOHEADER * pInfoHeader = NULL;
	BYTE * pData = NULL;
	CImageADesc * pDesc = NULL;
	try
	{
		CFile file(name, CFile::modeRead);
		pDesc = new CImageADesc;
		if(pDesc == NULL)
			throw 0;
		pDesc->m_strName = name;
		pDesc->m_Empty = false;
		pDesc->m_Handle = NULL;
		pDesc->m_cRef = 1;

		UINT length = file.GetLength();
		pFileData = (BITMAPFILEHEADER*)new BYTE[length];
		if(pFileData == NULL)
			throw 0;
		UINT nBytesRead = file.Read(pFileData, length);
		if(nBytesRead != length)
			throw 0;
		pInfoHeader = (BITMAPINFOHEADER*)((BYTE*)pFileData + sizeof(BITMAPFILEHEADER));
		if(pInfoHeader->biBitCount <= 8)
		{
			CString str;
			str.Format("File %s is not a truecolor or hicolor bitmap.", (LPCTSTR)name);
			AfxMessageBox(str);
			throw 0;
		}
		pData = (BYTE*)pInfoHeader + sizeof(BITMAPINFOHEADER);
		pDesc->m_Handle = RollAPI_MakeImage(pInfoHeader, pData);
		if(pDesc->m_Handle == NULL)
			throw 0;
		pDesc->m_Size = CSize(pInfoHeader->biHeight, pInfoHeader->biWidth);

		delete pFileData;
		pFileData = NULL; 

		if(g_plImages.AddTail(pDesc) == NULL)
			throw 0;
		return pDesc;
	}
	catch(...)
	{
		if(pDesc != NULL)
			DestroyImageDesc(pDesc);
		if(pFileData != NULL)
			delete pFileData;
	}
	return NULL;
}

static void DestroyImageDesc(CImageADesc * pDesc)
{
	ASSERT(pDesc);
	pDesc->m_cRef--;
	ASSERT(pDesc->m_cRef >= 0);
	if(pDesc->m_cRef == 0)
	{
		POSITION pos = g_plImages.Find(pDesc);
		if(pos != NULL)
			g_plImages.RemoveAt(pos);
		if(pDesc->m_Handle != NULL)
			RollAPI_DeleteImage(pDesc->m_Handle);
		delete pDesc;
	}
}

CImageA * CreateImage(CString &name)
{
	CImageADesc * pDesc = NULL;
	POSITION pos = g_plImages.GetHeadPosition();
	while(pos != NULL)
	{
		CImageADesc * pD = g_plImages.GetNext(pos);
		if(pD->m_strName == name)
		{
			pD->m_cRef++;
			pDesc = pD;
			break;
		}
	}
	CImageA * pImg = NULL;
	try
	{
		if(pDesc == NULL)
			pDesc = CreateImageDesc(name);
		if(pDesc == NULL)
			throw 0;

		pImg = new CImageA;
		pImg->m_pDesc = pDesc;
		pImg->m_Enabled = true;
		pImg->m_cDuration = 1;
		pImg->m_EnableInStatic = false;

		return pImg;
	}
	catch(...)
	{
		if(pDesc != NULL)
			DestroyImageDesc(pDesc);
	}
	return NULL;
}

CImageA * CreateEmptyImage(int size)
{
	CImageADesc * pDesc = new CImageADesc;
	if(pDesc == NULL)
		return NULL;

	pDesc->m_strName.Format("Empty: %d pixels", size);
	pDesc->m_Empty = true;
	pDesc->m_Size = CSize(size, size);
	pDesc->m_Handle = NULL;
	pDesc->m_cRef = 1;
	g_plImages.AddTail(pDesc);

	CImageA * pImg = new CImageA;
	if(pImg == NULL)
	{
		DestroyImageDesc(pDesc);
		return NULL;
	}

	pImg->m_pDesc = pDesc;
	pImg->m_Enabled = true;
	pImg->m_cDuration = 1;
	pImg->m_EnableInStatic = false;
	return pImg;
}

void DestroyImage(CImageA * pImg)
{
	ASSERT(pImg);
	if(pImg->m_pDesc != NULL)
		DestroyImageDesc(pImg->m_pDesc);
	delete pImg;
}

CRegionA * CreateRegion(CRect rect)
{
	ASSERT(!rect.IsRectEmpty());
	CRegionA * pReg = TestIntersection(rect);
	if(pReg != NULL)
		return NULL;
	pReg = new CRegionA;
	if(pReg == NULL)
		return NULL;
	pReg->m_Rect = rect;
	pReg->m_ScrollSpeed = 1.0;
	pReg->m_Type = CRegionA.RT_STATIC;
	pReg->m_Precision = 0;
	pReg->m_cRepeat = 1;
	pReg->m_Handle = NULL;
	g_plRegions.AddHead(pReg);
	return pReg;
}

void DestroyRegion(CRegionA * pReg)
{
	POSITION pos = pReg->m_plImages.GetHeadPosition();
	while(pos != NULL)
	{
		CImageA * pI = pReg->m_plImages.GetNext(pos);
		DestroyImage(pI);
	}
	g_plRegions.RemoveAt(g_plRegions.Find(pReg));
	delete pReg;
}

CRegionA * TestIntersection(CRect rect, CRegionA * pNotTest /* NULL */)
{
	CRect rc;
	rect.NormalizeRect();
	POSITION pos = g_plRegions.GetHeadPosition();
	while(pos != NULL)
	{	
		CRegionA * pR = g_plRegions.GetNext(pos);
		if(rc.IntersectRect(rect, pR->m_Rect) && (pR != pNotTest))
			return pR;
	}
	return NULL;
}

bool IsInsideScreen(CRect rect)
{
	rect.NormalizeRect();
	return (rect.left >= g_rcScreen.left) && (rect.right <= g_rcScreen.right)
		&& (rect.top >= g_rcScreen.top) && (rect.bottom <= g_rcScreen.bottom);
}

static int CountImages(CRegionA * pReg, int &size_x, int &size_y)
{
	int ret = 0;
	size_x = 0;
	size_y = 0;
	POSITION pos = pReg->m_plImages.GetHeadPosition();
	while(pos != NULL)
	{
		CImageA * pImg = pReg->m_plImages.GetNext(pos);
		if(pImg->m_Enabled)
		{
			if(!pImg->m_pDesc->m_Empty)
				ret++;
			size_x += pImg->m_pDesc->m_Size.cx;
			size_y += pImg->m_pDesc->m_Size.cy;
		}
	}
	return ret;
}

long StartPlayback(ScreenHandle &hScreen)//returns RollAPI error code (RollAPI_OK if succeed)
{
	if(hScreen != NULL)
	{
		RollAPI_ReleaseScreen(hScreen);
	}
	//it's better to allocate screen of minimal possible size, so let's find this size
	CRect rcScreenSize;
	POSITION pos = g_plRegions.GetHeadPosition();
	if(pos != NULL)
	{
		CRegionA * pReg = g_plRegions.GetNext(pos);
		rcScreenSize = pReg->m_Rect;
		while(pos != NULL)
		{
			pReg = g_plRegions.GetNext(pos);
			rcScreenSize |= pReg->m_Rect;
		}
	}
	else
		return RollAPI_OK;

	hScreen = RollAPI_AllocateScreen(rcScreenSize, g_DefaultBank);
	if(hScreen == NULL)
	{
		AfxMessageBox("Unable to allocate screen");
		return RollAPI_GetLastError();
	}
	ImageHandle hEmpty = NULL;
	POSITION reg_pos = g_plRegions.GetHeadPosition();
	while(reg_pos != NULL)
	{
		CRegionA * pReg = g_plRegions.GetNext(reg_pos);
		if(!pReg->m_plImages.IsEmpty())
		{
			if(pReg->m_Type == CRegionA::RT_STATIC)
			{
				CImageA * pI = pReg->m_plImages.GetHead();
				if(!pI->m_pDesc->m_Empty)
				{
					CPoint pos(pReg->m_Rect.left, pReg->m_Rect.top);
					pos -= rcScreenSize.TopLeft();
					RollAPI_ShowImage(hScreen, pI->m_pDesc->m_Handle, &pos, RollAPI_ShowDefault);
				}
			}
			else if(pReg->m_Type == CRegionA::RT_HSCROLL)
			{
				ASSERT(pReg->m_cRepeat >= 1);
				int size_x;
				int size_y;
				int counter = CountImages(pReg, size_x, size_y);
				if(counter == 0)
					break;
				size_x *= pReg->m_cRepeat;
				CPoint * pPt = new CPoint[counter*pReg->m_cRepeat];
				ImageHandle * phI = new ImageHandle[counter*pReg->m_cRepeat]; 
				int c = 0;
				int x = 0;
				for(int i=0; i < pReg->m_cRepeat; i++)
				{
					POSITION img_pos = pReg->m_plImages.GetHeadPosition();
					while(img_pos != NULL)
					{
						CImageA * pImg = pReg->m_plImages.GetNext(img_pos);
						if(pImg->m_Enabled)
						{
							if(!pImg->m_pDesc->m_Empty)
							{
								phI[c] = pImg->m_pDesc->m_Handle;
								pPt[c].x = x;
								pPt[c].y = 0;
								c++;
							}
							x += pImg->m_pDesc->m_Size.cx;
						}
					}
				}
				ImageHandle hI = RollAPI_ComposeImage(size_x, pReg->m_Rect.Height(), counter*pReg->m_cRepeat,
					pPt, phI, NULL);
				long time = -1;
				CRect rc = pReg->m_Rect - rcScreenSize.TopLeft();
				pReg->m_Handle = RollAPI_AllocateRegion(hScreen, rc,
					0.0, (float)pReg->m_ScrollSpeed, pReg->m_Precision,
					1, &hI, &time);
				RollAPI_DeleteImage(hI);
				delete pPt;
				delete phI;
			} 
			else if(pReg->m_Type == CRegionA::RT_VSCROLL)
			{
				ASSERT(pReg->m_cRepeat >= 1);
				int size_x;
				int size_y;
				int counter = CountImages(pReg, size_x, size_y);
				if(counter == 0)
					break;
				size_y *= pReg->m_cRepeat;
				CPoint * pPt = new CPoint[counter*pReg->m_cRepeat];
				ImageHandle * phI = new ImageHandle[counter*pReg->m_cRepeat]; 
				int c = 0;
				int y = 0;
				for(int i=0; i < pReg->m_cRepeat; i++)
				{
					POSITION img_pos = pReg->m_plImages.GetHeadPosition();
					while(img_pos != NULL)
					{
						CImageA * pImg = pReg->m_plImages.GetNext(img_pos);
						if(pImg->m_Enabled)
						{
							if(!pImg->m_pDesc->m_Empty)
							{
								phI[c] = pImg->m_pDesc->m_Handle;
								pPt[c].x = 0;
								pPt[c].y = y;
								c++;
							}
							y += pImg->m_pDesc->m_Size.cy;
						}
					}
				}
				ImageHandle hI = RollAPI_ComposeImage(pReg->m_Rect.Width(), size_y, counter*pReg->m_cRepeat,
					pPt, phI, NULL);
				long time = -1;
				CRect rc = pReg->m_Rect - rcScreenSize.TopLeft(); 
				pReg->m_Handle = RollAPI_AllocateRegion(hScreen, rc,
					(float)pReg->m_ScrollSpeed, 0.0, pReg->m_Precision,
					1, &hI, &time);
				RollAPI_DeleteImage(hI);
				delete pPt;
				delete phI;
			}
			else if(pReg->m_Type == CRegionA::RT_ANIMATED)
			{
				ASSERT(pReg->m_cRepeat >= 1);
				int counter = pReg->m_plImages.GetCount();
				if(counter == 0)
					break;
				ImageHandle * phI = new ImageHandle[counter]; 
				long * pTimes = new long[counter]; 
				POSITION img_pos = pReg->m_plImages.GetHeadPosition();
				int c = 0;
				while(img_pos != NULL)
				{
					CImageA * pImg = pReg->m_plImages.GetNext(img_pos);
					if(pImg->m_Enabled)
					{
						if(!pImg->m_pDesc->m_Empty)
							phI[c] = pImg->m_pDesc->m_Handle;
						else
							phI[c] = NULL;
						pTimes[c] = pImg->m_cDuration;
						c++;
					}
				}
				CRect rc = pReg->m_Rect - rcScreenSize.TopLeft();
				pReg->m_Handle = RollAPI_AllocateRegion(hScreen, rc,
					0.0, 0.0, pReg->m_Precision,
					counter, phI, pTimes);
				delete phI;
				delete pTimes;
			} 
			else
				ASSERT(0);
		}
	}
	if(RollAPI_PrepareScreenPlayback(hScreen) != RollAPI_OK)
	{
		AfxMessageBox("Unable to prepare playback");
		return RollAPI_GetLastError();
	}
	if(RollAPI_StartScreenPlayback(hScreen, RollAPI_ShowDefault) != RollAPI_OK)
	{
		AfxMessageBox("Unable to start playing");
		return RollAPI_GetLastError();
	}
	return RollAPI_OK;
}

long StopPlayback(ScreenHandle &hScreen)
{
	if(hScreen != NULL)
	{
		if(RollAPI_StopScreenPlayback(hScreen, RollAPI_ShowDefault) != RollAPI_OK)
			return RollAPI_GetLastError();
		if(RollAPI_ReleaseScreen(hScreen) != RollAPI_OK)
			return RollAPI_GetLastError();
		hScreen = NULL;
		//screen realised, so region handles are not valid, clear it now
		POSITION pos = g_plRegions.GetHeadPosition();
		while(pos != NULL)
		{
			CRegionA * pReg = g_plRegions.GetNext(pos);
			pReg->m_Handle = NULL;
		}
	}
	return RollAPI_OK;
}

long PausePlayback(ScreenHandle &hScreen)
{
	return RollAPI_PauseScreenPlayback(hScreen, RollAPI_ShowDefault);
}

long ContinuePlayback(ScreenHandle &hScreen)
{
	return RollAPI_UnpauseScreenPlayback(hScreen, RollAPI_ShowDefault);
}

long PauseRegionPlayback(CRegionA * pReg)
{
	return RollAPI_SetRegionSpeedScale(pReg->m_Handle, 0.0);
}

long HalfSpeedRegionPlayback(CRegionA * pReg)
{
	return RollAPI_SetRegionSpeedScale(pReg->m_Handle, 0.5);
}

long ContinueRegionPlayback(CRegionA * pReg)
{
	return RollAPI_SetRegionSpeedScale(pReg->m_Handle, 1.0);
}
