//////////////////////////////////////////////////////////////////////////////////
//																				//
//		Roller																	//
//		Test and Demo of RollAPI library										//
//		Copyright (C),1998 SoftLab-Nsk											//
//																				//
//		Author:		Serge Horun													//
//																				//
//		Last revised(86C):		Dec 25, 1998, Serge Horun						//
//																				//
//////////////////////////////////////////////////////////////////////////////////

#ifndef __ROLLBASE_H__
#define __ROLLBASE_H__

#include <afxtempl.h>

////////////////////////////////////////////////////////////////////
// structures and classes

struct CImageADesc;
struct CImageA;
struct CRegionA;
typedef CTypedPtrList<CPtrList, CImageADesc*> CImageDescPtrList;
typedef CTypedPtrList<CPtrList, CImageA*> CImagePtrList;
typedef CTypedPtrList<CPtrList, CRegionA*> CRegionPtrList;

struct CImageADesc
{
	CString m_strName;								//path and file name
	bool m_Empty;									//is image empty
	CSize m_Size;									//image size
	ImageHandle m_Handle;							//RollAPI corresponding image handle
	int m_cRef;										//counter of references
};

struct CImageA
{
	CImageADesc * m_pDesc;							//image descriptor
	bool m_Enabled;									//should image be shown or skipped
	int m_CtrlIndex;								//index in file list, valid only if image region is selected
	long m_cDuration;								//number of frames to show this image if image region is animated
	bool m_EnableInStatic;							//if region is static this image will be shown
};

struct CRegionA 
{
	CRect m_Rect;									//absolute screen coordinates
	float m_ScrollSpeed;							//scrolling speed
	enum {RT_VSCROLL, RT_HSCROLL, RT_STATIC, RT_ANIMATED} m_Type;//type of region
	CImagePtrList m_plImages;						//list of pointers to images
	int m_Precision;								//precision for playback, see RollAPI_XxPrecision
	int m_cRepeat;									//times to repeat
	RegionHandle m_Handle;							//handle to pause/unpause region during playback, not need to be released 
													//because all regions are automatically released when the whole screen is released
};

////////////////////////////////////////////////////////////////////
// variables

extern CImageDescPtrList g_plImages;
extern CRegionPtrList g_plRegions;
extern long g_DefaultBank;
extern CRect g_rcScreen;

/////////////////////////////////////////////////////////////////////
// functions

extern void InitScreenInfo();

extern CImageA * CreateImage(CString &name); 
extern CImageA * CreateEmptyImage(int size);
extern void DestroyImage(CImageA * pImg);

extern CRegionA * CreateRegion(CRect rect);
extern void DestroyRegion(CRegionA * pReg);
extern CRegionA * TestIntersection(CRect rect, CRegionA * pNotTest = NULL);
extern bool IsInsideScreen(CRect rect);

extern long StartPlayback(ScreenHandle &hScreen);			//returns RollAPI error code (RollAPI_OK if succeed)
extern long StopPlayback(ScreenHandle &hScreen);
extern long PausePlayback(ScreenHandle &hScreen);
extern long ContinuePlayback(ScreenHandle &hScreen);

extern long PauseRegionPlayback(CRegionA * pReg);
extern long HalfSpeedRegionPlayback(CRegionA * pReg);
extern long ContinueRegionPlayback(CRegionA * pReg);


#endif//__ROLLBASE_H__