//////////////////////////////////////////////////////////////////////////////////
//																				//
//		Roller																	//
//		Test and Demo of RollAPI library										//
//		Copyright (C),1998 SoftLab-Nsk											//
//																				//
//		Author:		Serge Horun													//
//																				//
//		Last revised(86C):		Dec 25, 1998, Serge Horun						//
//																				//
//////////////////////////////////////////////////////////////////////////////////

// ViewA.cpp : implementation file
//

#include "stdafx.h"
#include "RollAPI.h"
#include "RollBase.h"
#include "Roller.h"
#include "ViewA.h"
#include "RegionDialog.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CViewA

#define ResizeCornerSize 7

CViewA::CViewA()
{
	EnableAutomation();
	m_LBDown = false;
	m_pCurrent = NULL;
	m_Resize = false;
}

CViewA::~CViewA()
{
}

void CViewA::OnFinalRelease()
{
	// When the last reference for an automation object is released
	// OnFinalRelease is called.  The base class will automatically
	// deletes the object.  Add additional cleanup required for your
	// object before calling the base class.

	CWnd::OnFinalRelease();
}


BEGIN_MESSAGE_MAP(CViewA, CWnd)
	//{{AFX_MSG_MAP(CViewA)
	ON_WM_PAINT()
	ON_WM_LBUTTONDOWN()
	ON_WM_MOUSEMOVE()
	ON_WM_LBUTTONUP()
	ON_WM_CONTEXTMENU()
	ON_COMMAND(IDC_ADD_REGION, OnAddRegion)
	ON_COMMAND(IDC_EDIT_REGION, OnEditRegion)
	ON_COMMAND(IDC_REMOVE_REGION, OnRemoveRegion)
	ON_WM_LBUTTONDBLCLK()
	ON_COMMAND(IDC_REGION_CONTINUE, OnRegionContinue)
	ON_COMMAND(IDC_REGION_HALFSPEED, OnRegionHalfspeed)
	ON_COMMAND(IDC_REGION_PAUSE, OnRegionPause)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


BEGIN_DISPATCH_MAP(CViewA, CWnd)
	//{{AFX_DISPATCH_MAP(CViewA)
		// NOTE - the ClassWizard will add and remove mapping macros here.
	//}}AFX_DISPATCH_MAP
END_DISPATCH_MAP()

// Note: we add support for IID_IViewA to support typesafe binding
//  from VBA.  This IID must match the GUID that is attached to the 
//  dispinterface in the .ODL file.

// {507DC2A4-8A67-11D2-9E2A-00804883698B}
static const IID IID_IViewA =
{ 0x507dc2a4, 0x8a67, 0x11d2, { 0x9e, 0x2a, 0x0, 0x80, 0x48, 0x83, 0x69, 0x8b } };

BEGIN_INTERFACE_MAP(CViewA, CWnd)
	INTERFACE_PART(CViewA, IID_IViewA, Dispatch)
END_INTERFACE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CViewA message handlers

#define ViewABgColor RGB(90, 90, 255)
#define ViewAMoveColor RGB(0, 0, 0)
#define ViewANormalColor RGB(0, 0, 0)
#define ViewASelectColor RGB(0, 0, 0)

void CViewA::OnPaint() 
{
	CPaintDC dc(this); // device context for painting

	CPen * m_pOldPen = static_cast<CPen*>(dc.SelectStockObject(NULL_PEN));
	CPen NormalPen(PS_SOLID, 0, ViewANormalColor);
	CPen SelectPen(PS_SOLID, 3, ViewASelectColor);
	CPen MovePen(PS_DASH, 0, ViewAMoveColor);
	CRect rect;
	GetClientRect(&rect);
	dc.FillSolidRect(rect, ViewABgColor);
	CPoint points[5];
	CRegionA * pCurr;
	POSITION pos = g_plRegions.GetHeadPosition();
	while(pos != NULL)
	{
		pCurr = g_plRegions.GetNext(pos);
		points[0] = CPoint(pCurr->m_Rect.left, pCurr->m_Rect.top);
		points[1] = CPoint(pCurr->m_Rect.left, pCurr->m_Rect.bottom);
		points[2] = CPoint(pCurr->m_Rect.right, pCurr->m_Rect.bottom);
		points[3] = CPoint(pCurr->m_Rect.right, pCurr->m_Rect.top);
		RecalcPointToScreen(points[0]);
		RecalcPointToScreen(points[1]);
		RecalcPointToScreen(points[2]);
		RecalcPointToScreen(points[3]);
		points[4] = points[0];
		if(m_pCurrent == pCurr)
		{
			if(m_LBDown)
				dc.SelectObject(&MovePen);
			else
				dc.SelectObject(&SelectPen);
		}
		else
			dc.SelectObject(&NormalPen);
		dc.Polyline(points, 5);
		if(m_pCurrent == pCurr)
		{
			CPoint pt(pCurr->m_Rect.right, pCurr->m_Rect.bottom);
			CPoint pt2(pCurr->m_Rect.left, pCurr->m_Rect.top);
			RecalcPointToScreen(pt);
			RecalcPointToScreen(pt2);
			if(((pt.x - pt2.x) > ResizeCornerSize) && ((pt.y - pt2.y) > ResizeCornerSize))
			{
				dc.SelectObject(&NormalPen);
				dc.MoveTo(pt.x - ResizeCornerSize, pt.y);
				dc.LineTo(pt.x - ResizeCornerSize, pt.y - ResizeCornerSize);
				dc.LineTo(pt.x, pt.y - ResizeCornerSize);
			}
		}
	}
	if(m_LBDown && (m_pCurrent == NULL))
	{
		dc.SelectObject(&MovePen);
		points[0] = CPoint(m_rcNewReg.left, m_rcNewReg.top);
		points[1] = CPoint(m_rcNewReg.left, m_rcNewReg.bottom);
		points[2] = CPoint(m_rcNewReg.right, m_rcNewReg.bottom);
		points[3] = CPoint(m_rcNewReg.right, m_rcNewReg.top);
		RecalcPointToScreen(points[0]);
		RecalcPointToScreen(points[1]);
		RecalcPointToScreen(points[2]);
		RecalcPointToScreen(points[3]);
		points[4] = points[0];
		dc.Polyline(points, 5);
	}
	dc.SelectObject(m_pOldPen);
}

void CViewA::OnLButtonDown(UINT nFlags, CPoint point) 
{
	CPoint p2 = point;
	RecalcPointToReg(p2);
	CRegionA * pCurr = HitTest(p2);
	if(pCurr != m_pCurrent)
	{
		m_pCurrent = pCurr;
		GetOwner()->SendMessage(WM_COMMAND, IDC_UPDATE_REG_INFO);
	}
	if(pCurr != NULL)
	{
		CPoint pt(pCurr->m_Rect.right, pCurr->m_Rect.bottom);
		CPoint pt2(pCurr->m_Rect.left, pCurr->m_Rect.top);
		RecalcPointToScreen(pt);
		RecalcPointToScreen(pt2);
		if(((pt.x - pt2.x) > ResizeCornerSize) && ((pt.y - pt2.y) > ResizeCornerSize))
		{
			 m_Resize = (point.x >= (pt.x - ResizeCornerSize)) && (point.y >= (pt.y - ResizeCornerSize));
		}
	}
	m_ptCurr = p2;
	m_LBDown = true;
	SetCapture();
	if(m_pCurrent == NULL)
	{
		m_rcNewReg.SetRect(p2.x, p2.y, p2.x, p2.y);
	}
	InvalidateRect(NULL, FALSE);
}

CRegionA * CViewA::HitTest(CPoint point)
{
	CRegionA * pCurr;
	POSITION pos = g_plRegions.GetHeadPosition();
	while(pos != NULL)
	{
		pCurr = g_plRegions.GetNext(pos);
		if(pCurr->m_Rect.PtInRect(point))
			return pCurr;
	}
	return NULL;
}

void CViewA::RecalcPointToReg(CPoint &point)
{
	CRect rect;
	GetClientRect(&rect);
	point.x = point.x * g_rcScreen.Width() / rect.right;
	point.y = point.y * g_rcScreen.Height() / rect.bottom;
}

void CViewA::OnMouseMove(UINT nFlags, CPoint point) 
{
	if(m_LBDown)
	{
		CPoint p2 = point;
		RecalcPointToReg(p2);
		CPoint p3 = p2;
		if(m_pCurrent == NULL)
		{
			CRect rect = CRect(m_rcNewReg.left, m_rcNewReg.top, p2.x, p2.y);
			if((TestIntersection(rect) == NULL) && IsInsideScreen(rect))
				m_rcNewReg = rect;
		}
		else if(!m_Resize)
		{
			CRect rc = m_pCurrent->m_Rect;
			p2 -= m_ptCurr;
			rc += p2;
			if((TestIntersection(rc, m_pCurrent) == NULL) && IsInsideScreen(rc))
				m_pCurrent->m_Rect = rc;
		}
		else
		{
			CRect rc(m_pCurrent->m_Rect.left, m_pCurrent->m_Rect.top, p2.x, p2.y);
			if((p2.x > m_pCurrent->m_Rect.left) && (p2.y > m_pCurrent->m_Rect.top)
				&& (TestIntersection(rc, m_pCurrent) == NULL) && IsInsideScreen(rc))
				m_pCurrent->m_Rect = rc;
		}
		m_ptCurr = p3;
		InvalidateRect(NULL, FALSE);
	}
}

void CViewA::OnLButtonUp(UINT nFlags, CPoint point) 
{
	ReleaseCapture();
	if(m_LBDown)
	{
		m_LBDown = false;
		m_Resize = false;
		m_rcNewReg.NormalizeRect();
		if((m_pCurrent == NULL) && !m_rcNewReg.IsRectEmpty())
		{
			m_pCurrent = CreateRegion(m_rcNewReg);
			GetOwner()->SendMessage(WM_COMMAND, IDC_UPDATE_REG_INFO);
		}
		else
		{
		}
		InvalidateRect(NULL, FALSE);
	}
}

void CViewA::RecalcPointToScreen(CPoint & point)
{
	CRect rect;
	GetClientRect(&rect);
	point.x = point.x * rect.right / g_rcScreen.Width();
	point.y = point.y * rect.bottom / g_rcScreen.Height();
}

void CViewA::OnContextMenu(CWnd* pWnd, CPoint point) 
{
	if(pWnd == this)
	{
		CPoint p2 = point;
		ScreenToClient(&p2);
		RecalcPointToReg(p2);
		CRegionA * pCurr = HitTest(p2);
		if(pCurr != m_pCurrent)
		{
			m_pCurrent = pCurr;
			GetOwner()->SendMessage(WM_COMMAND, IDC_UPDATE_REG_INFO);
			InvalidateRect(NULL, FALSE);
		}

		CMenu menu;
		menu.LoadMenu(IDR_MENU_POPUP);
		CMenu *pSubMenu = menu.GetSubMenu(1);
		UINT todo = MF_BYCOMMAND | ((m_pCurrent != NULL)?MF_ENABLED:MF_GRAYED);
		UINT playback = MF_BYCOMMAND | (((m_pCurrent != NULL) && (m_pCurrent->m_Handle != NULL))?MF_ENABLED:MF_GRAYED);
		pSubMenu->EnableMenuItem(IDC_REMOVE_REGION, todo);
		pSubMenu->EnableMenuItem(IDC_EDIT_REGION, todo);
		pSubMenu->EnableMenuItem(IDC_REGION_PAUSE, playback);
		pSubMenu->EnableMenuItem(IDC_REGION_HALFSPEED, playback);
		pSubMenu->EnableMenuItem(IDC_REGION_CONTINUE, playback);
		pSubMenu->TrackPopupMenu(TPM_LEFTALIGN, point.x, point.y, this);
		menu.DestroyMenu();
	}
	else
	{
//		DefWindowProc(WM_CONTEXTMENU, (WPARAM)pWnd, reinterpret_cast<LPARAM>(point));
	}
}

void CViewA::OnAddRegion() 
{
	CRegionDialog dlg("Add region", this);
	dlg.m_Top = 0;
	dlg.m_Left = 0;
	dlg.m_Height  = 100;
	dlg.m_Width = 100;
	dlg.m_ScreenSize = GetScreenSizeString();
	if(dlg.DoModal() == IDOK)
	{
		m_pCurrent = CreateRegion(CRect(dlg.m_Left, dlg.m_Top, dlg.m_Left+dlg.m_Width, dlg.m_Top+dlg.m_Height));
		GetOwner()->SendMessage(WM_COMMAND, IDC_UPDATE_REG_INFO);
		InvalidateRect(NULL, FALSE);
	}
}

CString CViewA::GetScreenSizeString()
{
	CString str;
	str.Format("%dx%d pix.", g_rcScreen.Width(), g_rcScreen.Height());
	return str;
}

void CViewA::OnEditRegion()
{
	ASSERT(m_pCurrent);
	CRegionDialog dlg("Edit region", this);
	dlg.m_Top = m_pCurrent->m_Rect.top;
	dlg.m_Left = m_pCurrent->m_Rect.left;
	dlg.m_Height  = m_pCurrent->m_Rect.Height();
	dlg.m_Width = m_pCurrent->m_Rect.Width();
	dlg.m_ScreenSize = GetScreenSizeString();
	if(dlg.DoModal() == IDOK)
	{
		m_pCurrent->m_Rect = CRect(dlg.m_Left, dlg.m_Top, dlg.m_Left+dlg.m_Width, dlg.m_Top+dlg.m_Height);
		GetOwner()->SendMessage(WM_COMMAND, IDC_UPDATE_REG_INFO);
		InvalidateRect(NULL, FALSE);
	}
}

void CViewA::OnRemoveRegion() 
{
	ASSERT(m_pCurrent);
	if(AfxMessageBox("Remove selected region?", MB_YESNO) == IDYES)
	{
		DestroyRegion(m_pCurrent);
		m_pCurrent = NULL;
		GetOwner()->SendMessage(WM_COMMAND, IDC_UPDATE_REG_INFO);
		InvalidateRect(NULL, FALSE);
	}
}

void CViewA::OnLButtonDblClk(UINT nFlags, CPoint point) 
{
	if(m_pCurrent != NULL)
		SendMessage(WM_COMMAND, IDC_EDIT_REGION);
	
	CWnd::OnLButtonDblClk(nFlags, point);
}

void CViewA::OnRegionContinue() 
{
	ContinueRegionPlayback(m_pCurrent);
}

void CViewA::OnRegionHalfspeed() 
{
	HalfSpeedRegionPlayback(m_pCurrent);
}

void CViewA::OnRegionPause() 
{
	ASSERT(m_pCurrent);
	PauseRegionPlayback(m_pCurrent);
}
